@@ -2,6 +2,8 @@
 {-# LANGUAGE TypeFamilies #-}
 {-# LANGUAGE DataKinds #-}
 {-# LANGUAGE LambdaCase #-}
+{-# LANGUAGE TypeApplications #-}
+
 -- -*-haskell-*-
 --  GIMP Toolkit (GTK) CustomStore TreeModel
 --
@@ -67,11 +69,13 @@ import Foreign.Marshal (fromBool)
 import Foreign.Storable (peek, poke, peekByteOff)
 import System.IO.Unsafe (unsafePerformIO)
 import Data.GI.Base.BasicTypes
-       (ManagedPtr(..), GObject(..), GType, CGType(..), gtypeToCGType,
-        gtypeInt, gtypeBoolean, gtypeString, gtypeInvalid)
+       (ManagedPtr(..), GObject, TypedObject(..),
+        GType, CGType(..), gtypeToCGType)
+import Data.GI.Base.GType (gtypeInt, gtypeBoolean, gtypeString, gtypeInvalid)
 import Data.GI.Base.BasicConversions (gflagsToWord, withTextCString)
 import Data.GI.Base.ManagedPtr (newObject, withManagedPtr, newManagedPtr_)
 import Data.GI.Base.GValue (GValue(..))
+import Data.GI.Base.Overloading (HasParentTypes, ParentTypes)
 import GI.GObject (Object)
 import GI.GdkPixbuf.Objects (Pixbuf(..))
 import GI.Gtk.Flags (TreeModelFlags(..))
@@ -108,13 +112,14 @@ treeIterOverwrite iterOut iterIn = do
 --   models 'Data.GI.Gtk.ModelView.ListStore.ListStore' or
 --   'Data.GI.Gtk.ModelView.TreeStore.TreeStore'.
 newtype CustomStore private row = CustomStore (ManagedPtr (CustomStore private row))
-instance IsTreeModel (CustomStore private row)
+
+instance HasParentTypes (CustomStore private row)
+type instance ParentTypes (CustomStore private row) = '[ TreeModel ]
+
+instance TypedObject (CustomStore private row) where
+  glibType = glibType @TreeModel
 
 instance GObject (CustomStore private row) where
-#if !MIN_VERSION_haskell_gi_base(0,20,1)
-    gobjectIsInitiallyUnowned _ = False
-#endif
-    gobjectType _ = gobjectType (undefined :: TreeModel)
 
 -- | Type synonym for viewing the store as a set of columns.
 type ColumnMap row = IORef [ColumnAccess row]
@@ -314,7 +319,7 @@ caToGType (CAPixbuf _) = gtypePixbuf
 caToGType CAInvalid = gtypeInt -- to avoid warnings of functions that iterate through all columns
 
 gtypePixbuf :: GType
-gtypePixbuf = unsafePerformIO $ gobjectType (undefined :: Pixbuf)
+gtypePixbuf = unsafePerformIO $ glibType @Pixbuf
 {-# NOINLINE gtypePixbuf #-}
 
 treeModelIfaceGetColumnType_static :: StablePtr (CustomStoreImplementation model row) -> CInt -> IO CGType
