@@ -3,6 +3,8 @@
 {-# LANGUAGE DataKinds #-}
 {-# LANGUAGE LambdaCase #-}
 {-# LANGUAGE ScopedTypeVariables #-}
+{-# LANGUAGE TypeApplications #-}
+
 -- -*-haskell-*-
 --  GIMP Toolkit (GTK) CustomStore TreeModel
 --
@@ -75,8 +77,9 @@ import Data.IORef
 import Foreign.ForeignPtr (ForeignPtr)
 
 import Data.GI.Base.BasicTypes
-       (ManagedPtr(..), GObject(..), GObject)
+       (TypedObject(..), ManagedPtr(..), GObject)
 import Data.GI.Base.ManagedPtr (withManagedPtr)
+import Data.GI.Base.Overloading (HasParentTypes, ParentTypes)
 
 import Data.GI.Gtk.ModelView.Types
 import Data.GI.Gtk.ModelView.CustomStore
@@ -132,13 +135,13 @@ newtype ForestStore a = ForestStore (ManagedPtr (CustomStore (IORef (Store a)) a
 mkForestStore :: CustomStore (IORef (Store a)) a -> ForestStore a
 mkForestStore (CustomStore ptr) = ForestStore ptr
 
-instance IsTreeModel (ForestStore a)
+instance HasParentTypes (ForestStore a)
+type instance ParentTypes (ForestStore a) = '[TreeModel]
 
-instance GObject (ForestStore a) where
-#if !MIN_VERSION_haskell_gi_base(0,20,1)
-    gobjectIsInitiallyUnowned _ = False
-#endif
-    gobjectType _ = gobjectType (undefined :: TreeModel)
+instance TypedObject (ForestStore a) where
+  glibType = glibType @TreeModel
+
+instance GObject (ForestStore a)
 
 instance IsTypedTreeModel ForestStore
 
